<?php

class InXpress_InXpress_Model_Carrier_DHLExpress
    extends Mage_Shipping_Model_Carrier_Abstract
    implements Mage_Shipping_Model_Carrier_Interface
{

    protected $_code = 'inxpress_dhl_express';

    public function collectRates(Mage_Shipping_Model_Rate_Request $request)
    {
        if (!$this->getConfigFlag('active')) {
            Mage::log("foobar", null, 'shipping.log');
            return false;
        }
        $shippingPrice = 0;
        $products = array();
        if ($request->getAllItems()) {
            foreach ($request->getAllItems() as $item) {
                if ($item->getProduct()->isVirtual() || $item->getParentItem()) {
                    continue;
                }

                if ($item->getHasChildren() && $item->isShipSeparately()) {
                    foreach ($item->getChildren() as $child) {
                        if (!$child->getFreeShipping() && !$child->getProduct()->isVirtual()) {
                            array_push($products, $child->toArray());
                        }
                    }
                } else {
                    array_push($products, array(
                        "id" => $item->getProductId(),
                        "sku" => $item->getSku(),
                        "name" => $item->getName(),
                        "weight" => $this->itemWeight($item),
                        "quantity" => $item->getQty()
                    ));
                }
            }

            $origin = array(
                    "name" => "",
                    "address1" => "",
                    "address2" => "",
                    "city" => "",
                    "province" => "",
                    "phone" => "",
                    "country" => $request->getOriginCountryId(),
                    "postal_code" => $request->getOriginPostcode()
            );

            $destination = array(
                "name" => "",
                "address1" => "",
                "address2" => "",
                "city" => "",
                "province" => "",
                "phone" => "",
                "country" => $request->getDestCountryId(),
                "postal_code" => $request->getDestPostcode()
            );

            $gateway = Mage::getStoreConfig('carriers/inxpress_dhl_express/gateway');
            $account = Mage::getStoreConfig('carriers/inxpress_dhl_express/customer_number');

            $price = $this->calcRate($account, $gateway, $products, $origin, $destination);

            if ($price) {
                $shippingPrice = $price['price'];
            } else {
                return false;
            }
        }
        $result = Mage::getModel('shipping/rate_result');
        $shippingPrice = $this->getFinalPriceWithHandlingFee($shippingPrice);
        if ($shippingPrice != 0) {

            $method = Mage::getModel('shipping/rate_result_method');

            $method->setCarrier($this->_code);
            $method->setCarrierTitle(Mage::getStoreConfig('carriers/inxpress_dhl_express/title'));

            $method->setMethod('standard');
            $method->setMethodTitle(Mage::getStoreConfig('carriers/inxpress_dhl_express/title') . ' ( Transit Days: ' . $price['days'] . ' )');

            $method->setPrice($shippingPrice);
            $method->setCost($shippingPrice);

            $result->append($method);
        }

        return $result;
    }

    public function itemWeight($item)
    {
        $weight_in_uom = floatval($item->getWeight());
        $weight_unit = Mage::getStoreConfig('carriers/inxpress_ups/weight_unit');

        switch ( $weight_unit ) {
            case 'lbs':
                $weight = $weight_in_uom * 453.5920;
                break;
            case 'kgs':
                $weight = $weight_in_uom * 1000;
                break;
            default:
                $weight = $weight_in_uom * 453.5920;
        }

        return $weight;
    }

    public function calcRate($account, $gateway, $products, $origin, $destination)
    {
        $store_id = Mage::getStoreConfig('system/inxpress/store_id');
        $url = "https://api.inxpressapps.com/carrier/v1/stores/" . $store_id . "/rates";

        $request = json_encode(array(
            "account" => $account,
            "gateway" => $gateway,
            "services" => array(array(
                "carrier" => "DHL",
                "service" => "DHL Express"
            )),
	        "origin" => $origin,
            "destination" => $destination,
            "products" => $products
        ));

        Mage::log($url, null, 'shipping.log');
        Mage::log($request, null, 'shipping.log');

        try {
            $curl = new Varien_Http_Adapter_Curl();
            $curl->setConfig(array(
                'timeout'   => 15
            ));
            $curl->write(Zend_Http_Client::POST, $url, '1.1', array('Content-Type: text/json'), $request);

            $res = $curl->read();

            $cod = Zend_Http_Response::extractCode($res);
            $response = '';
            if ($cod == 200) {
                Mage::log($res, null, 'shipping.log');
                $tmp = preg_split('/^\r?$/m', $res, 2);
                $response = trim($tmp[1]);
                $responseArray = json_decode($response, true);
            } else {
                Mage::log("$cod", null, 'shipping.log');
                Mage::log($cod, null, 'shipping.log');
            }

            # Close Call
            $curl->close();
        } catch (Exception $e) {
            Mage::logException($e);
        }

        Mage::log($responseArray, null, 'shipping.log');

	    if (isset($responseArray["rates"][0]["total_price"])) {
            $response = array();
            $response['price'] = $responseArray["rates"][0]["total_price"] / 100;
            $response['days'] = $responseArray["rates"][0]["display_sub_text"];
            return $response;
        } else {
            Mage::log("total_price is not set", null, 'shipping.log');
            return false;
        }
    }

    /**
     * Get allowed shipping methods
     *
     * @return array
     */
    public function getAllowedMethods()
    {
        return array(
            'standard' => 'Standard',
        );
    }
}
