<?php

class InXpress_InXpress_Model_Carrier_UPS
    extends Mage_Shipping_Model_Carrier_Abstract
    implements Mage_Shipping_Model_Carrier_Interface
{

    protected $_code = 'inxpress_ups';

    public function collectRates(Mage_Shipping_Model_Rate_Request $request)
    {
        if (!$this->getConfigFlag('active')) {
            Mage::log("foobar", null, 'shipping.log');
            return false;
        }
        $shippingPrice = 0;
        $products = array();
        if ($request->getAllItems()) {
            $products = $this->loadProducts($request, $products);

            $origin = $this->getOrigin();

            $destination = $this->getDestination($request);

            $gateway = Mage::getStoreConfig('carriers/inxpress_ups/gateway');
            $account = Mage::getStoreConfig('carriers/inxpress_ups/customer_number');
            $result = Mage::getModel('shipping/rate_result');

            $rates = $this->calcRate($account, $gateway, $products, $origin, $destination);

            if ($rates) {
                foreach($rates as $rate) {
                    if ($rate) {
                        $shippingPrice = $this->getFinalPriceWithHandlingFee($rate['price']);
                    } else {
                        return false;
                    }

                    if ($shippingPrice != 0) {
                        $method = Mage::getModel('shipping/rate_result_method');

                        $method->setCarrier($this->_code);
                        $method->setCarrierTitle(Mage::getStoreConfig('carriers/inxpress_ups/title'));

                        $method->setMethod('standard');
                        $method->setMethodTitle($rate['service']);

                        $method->setPrice($shippingPrice);
                        $method->setCost($shippingPrice);

                        $result->append($method);

                    }
                }
            }

        }

        return $result;
    }

    protected function getOrigin()
    {
        return array(
            "name" => "",
            "address1" => "",
            "address2" => "",
            "city" => Mage::getStoreConfig('shipping/origin/city'),
            "province" => "",
            "phone" => "",
            "country" => Mage::getStoreConfig('shipping/origin/country_id'),
            "postal_code" => Mage::getStoreConfig('shipping/origin/postcode')
        );
    }

    protected function getDestination($request)
    {
        return array(
            "name" => "",
            "address1" => "",
            "address2" => "",
            "city" => $request->getDestCity(),
            "province" => "",
            "phone" => "",
            "country" => $request->getDestCountryId(),
            "postal_code" => $request->getDestPostcode()
        );
    }

    protected function loadProducts($request, $products)
    {
        foreach ($request->getAllItems() as $item) {
            if ($item->getProduct()->isVirtual() || $item->getParentItem()) {
                continue;
            }

            if ($item->getHasChildren() && $item->isShipSeparately()) {
                foreach ($item->getChildren() as $child) {
                    if (!$child->getFreeShipping() && !$child->getProduct()->isVirtual()) {
                        array_push($products, $child->toArray());
                    }
                }
            } else {
                array_push($products, array(
                    "id" => $item->getProductId(),
                    "sku" => $item->getSku(),
                    "name" => $item->getName(),
                    "weight" => $this->itemWeight($item),
                    "quantity" => $item->getQty()
                ));
            }
        }

        return $products;
    }

    public function itemWeight($item)
    {
        $weight_in_uom = floatval($item->getWeight());
        $weight_unit = Mage::getStoreConfig('carriers/inxpress_ups/weight_unit');

        switch ( $weight_unit ) {
            case 'lbs':
                $weight = $weight_in_uom * 453.5920;
                break;
            case 'kgs':
                $weight = $weight_in_uom * 1000;
                break;
            default:
                $weight = $weight_in_uom * 453.5920;
        }

        return $weight;
    }

    public function calcRate($account, $gateway, $products, $origin, $destination)
    {
        $store_id = Mage::getStoreConfig('system/inxpress/store_id');
        $url = "https://api.inxpressapps.com/carrier/v1/stores/" . $store_id . "/rates";

        $request = json_encode(array(
            "account" => $account,
            "gateway" => $gateway,
            "services" => array(array(
                "carrier" => "UPS",
                "service" => "DHL Express"
            )),
	          "origin" => $origin,
            "destination" => $destination,
            "products" => $products
        ));

        Mage::log($url, null, 'shipping.log');
        Mage::log($request, null, 'shipping.log');

        try {
            $curl = new Varien_Http_Adapter_Curl();
            $curl->setConfig(array(
                'timeout'   => 15
            ));
            $curl->write(Zend_Http_Client::POST, $url, '1.1', array('Content-Type: text/json'), $request);

            $res = $curl->read();

            $cod = Zend_Http_Response::extractCode($res);
            $response = '';
            if ($cod == 200) {
                Mage::log($res, null, 'shipping.log');
                $tmp = preg_split('/^\r?$/m', $res, 2);
                $response = trim($tmp[1]);
                $responseArray = json_decode($response, true);
            } else {
                Mage::log("$cod", null, 'shipping.log');
                Mage::log($cod, null, 'shipping.log');
            }

            # Close Call
            $curl->close();
        } catch (Exception $e) {
            Mage::logException($e);
        }

        Mage::log($responseArray, null, 'shipping.log');

        if (isset($responseArray["rates"][0]["total_price"])) {
          $responses = array();
          foreach($responseArray["rates"] as $rate) {
              $response = array();

              $response['price'] = $rate["total_price"] / 100;
              $response['days'] = $rate["display_sub_text"];
              $response['service'] = $rate["display_text"];
              array_push($responses, $response);
          }
          return $responses;
      } else {
          Mage::log("No Rates returned", null, 'shipping.log');
          return false;
      }
    }

    /**
     * Get allowed shipping methods
     *
     * @return array
     */
    public function getAllowedMethods()
    {
        return array(
            'standard' => 'Standard',
        );
    }
}
